(*
    Stack Quizzer - A tool for memorizing decks of cards
    Deck editor
    Copyright (c) 2003 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Editor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
  Cards, Deck;

type
  TFormEditor = class(TForm)
    Label1: TLabel;
    PaintDeck: TPaintBox;
    ButtonOk: TButton;
    ButtonCancel: TButton;
    procedure FormCreate(Sender: TObject);
    procedure PaintDeckPaint(Sender: TObject);
    procedure PaintDeckMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintDeckStartDrag(Sender: TObject;
      var DragObject: TDragObject);
    procedure PaintDeckDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure PaintDeckEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PaintDeckMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    FBitmapDeck: TBitmap;
    FSourceDeck: TDeck;
    FDeck: TDeck;
    FSequenceIndex: integer;
    // Drag and drop card movement
    FDragging: boolean;
    FDragSourcePos: integer;
    FDragTargetPos: integer;

    function GetCardAtPos( X, Y: integer ): integer;
    procedure DrawDeckOnBitmap( const D: TDeck );

  public
    { Public declarations }
    procedure SetDeck( const D: TDeck );
    procedure GetDeck( var D: TDeck );
  end;

var
  FormEditor: TFormEditor;

implementation

uses
  Help;

const
  HorizontalMargin = 4;
  VerticalMargin = 8;
  HalfDeckMargin = 16;
  CardMargin = 16;

{$R *.DFM}

procedure TFormEditor.FormCreate(Sender: TObject);
begin
  FDragging := false;
  FSequenceIndex := 1;

  PaintDeck.Width :=
    HorizontalMargin + // Left
    25*CardMargin + cardGetDefaultWidth + // Half deck
    HorizontalMargin; // Right

  PaintDeck.Height :=
    VerticalMargin + // Top
    cardGetDefaultHeight + HalfDeckMargin + cardGetDefaultHeight + // Deck
    VerticalMargin; // Bottom

  FBitmapDeck := TBitmap.Create;
  FBitmapDeck.Width := PaintDeck.Width;
  FBitmapDeck.Height := PaintDeck.Height;
  FBitmapDeck.Canvas.Brush.Style := bsSolid;
  FBitmapDeck.Canvas.Brush.Color := Color; // Current window color
  FBitmapDeck.Canvas.FillRect( Rect(0, 0, FBitmapDeck.Width, FBitmapDeck.Height) );

  ClientWidth := PaintDeck.Left*2 + PaintDeck.Width;
  ClientHeight := PaintDeck.Top + PaintDeck.Height + PaintDeck.Left + // Use left margin for bottom too
    ButtonOk.Height + 8;
end;

procedure TFormEditor.DrawDeckOnBitmap( const D: TDeck );
var
  I, X: integer;
procedure DrawCard( I, X, Y: integer );
begin
  if( FDragging )and( FDragTargetPos = I ) then begin
    cardDrawEx( FBitmapDeck.Canvas, D[I+1], X, Y, 2, clWhite );
  end
  else begin
    cardDraw( FBitmapDeck.Canvas, D[I+1], X, Y );
  end;
end;
begin
  for I:=0 to 25 do begin
    X := HorizontalMargin + I*CardMargin;
    DrawCard( I, X, VerticalMargin );
    DrawCard( I+26, X, VerticalMargin + cardGetDefaultHeight + HalfDeckMargin );
  end;
end;

procedure TFormEditor.PaintDeckPaint(Sender: TObject);
var
  D: TDeck;
begin
  if( not FDragging ) then begin
    DrawDeckOnBitmap( FDeck );
  end
  else begin
    D := FDeck;
    deckMoveCard( D, FDragSourcePos+1, FDragTargetPos+1 );
    DrawDeckOnBitmap( D );
  end;
  PaintDeck.Canvas.Draw( 0, 0, FBitmapDeck );
end;

procedure TFormEditor.SetDeck( const D: TDeck );
begin
  FSourceDeck := D;
  FDeck := FSourceDeck;
  FSequenceIndex := 1;
end;

procedure TFormEditor.GetDeck( var D: TDeck );
begin
  D := FDeck;
end;

function TFormEditor.GetCardAtPos( X, Y: integer ): integer;
var
  Base: integer;
begin
  Result := -1;

  // Exit if horizonal position not in range
  if( X < HorizontalMargin )or( X >= PaintDeck.Width-HorizontalMargin ) then
    Exit;

  Base := 0;
  if( Y > VerticalMargin + cardGetDefaultHeight + HalfDeckMargin ) then begin
    // Second half of the deck
    Base := 26;
    Dec( Y, cardGetDefaultHeight + HalfDeckMargin );
  end;

  if( Y >= VerticalMargin )and( Y < VerticalMargin+cardGetDefaultHeight ) then begin
    Dec( X, HorizontalMargin );
    if( X < 25*CardMargin ) then
      Result := X div CardMargin
    else
      Result := 25;
  end;

  Inc( Result, Base );
end;

procedure TFormEditor.PaintDeckMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if( FDragging ) then
    FDragTargetPos := GetCardAtPos( X, Y )
  else
    FDragSourcePos := GetCardAtPos( X, Y );
end;

procedure TFormEditor.PaintDeckStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  FDragging := FDragSourcePos >= 0;
end;

procedure TFormEditor.PaintDeckDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := false;
  if( FDragging ) then begin
    FDragTargetPos := GetCardAtPos( X, Y );
    Accept := FDragTargetPos >= 0;
    PaintDeckPaint(Sender);
  end;
end;

procedure TFormEditor.PaintDeckEndDrag(Sender, Target: TObject; X,
  Y: Integer);
begin
  if( FDragging )and( FDragTargetPos >= 0 ) then begin
    deckMoveCard( FDeck, FDragSourcePos+1, FDragTargetPos+1 );
    FSequenceIndex := FDragTargetPos+1;
    if( FSequenceIndex < CardsInDeck ) then
      Inc( FSequenceIndex );
  end;
  FDragging := false;
  PaintDeckPaint(Sender);
end;

procedure TFormEditor.FormShow(Sender: TObject);
begin
  ActiveControl := ButtonOk;
end;

procedure TFormEditor.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if( Shift = [] )and( Key = VK_F1 ) then begin
    OpenHelp( Self.Handle, 'editor' );
  end;
end;

procedure TFormEditor.PaintDeckMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Card: integer;
begin
  if( Button <> mbRight )or( Shift <> [ssRight] ) then
    Exit;
  Card := GetCardAtPos( X, Y ) + 1;
  if( Card > FSequenceIndex ) then begin
    deckMoveCard( FDeck, Card, FSequenceIndex );
    if( FSequenceIndex < CardsInDeck ) then
      Inc( FSequenceIndex );
    PaintDeckPaint(Sender);
  end;
end;

end.
